########################### PSQF 6270 Example 2a and 2b using R #################################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

#if (!require("expss")) install.packages("expss")
#library(expss) # To add variable and value labels, sorting

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add descriptive summary functions

if (!require("polycor")) install.packages("polycor")
library(polycor) # To get polychoric and polyserial correlations

if (!require("multcomp")) install.packages("multcomp")
library(multcomp) # To do glht linear combinations in GLMs

if (!require("prediction")) install.packages("prediction")
library(prediction) # To get predicted values like Stata does

if (!require("DescTools")) install.packages("DescTools")
library(DescTools) # To get pseudo-R2 and odds ratios for GLMs

if (!require("VGAM")) install.packages("VGAM")
library(VGAM) # To fit non-proportional odds models


######################################################################################
#####               BEGIN DATA MANIPULATION OF EXAMPLE 2a AND 2b                 #####
#####                   CHANGE "filesave" TO YOUR FOLDER PATH                    #####
######################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/24_PSQF6270/PSQF6270_Example2ab/"
filename = "Excel_Example2_Data.xlsx"
setwd(dir=filesave)

# Import Example2a and Example 2b Excel data
Example2 = read_excel(path=paste0(filesave,filename))
# Convert to data frame without labels to use for analysis
Example2 = as.data.frame(Example2)

# Create ID variable
Example2$PersonID = seq.int(nrow(Example2)) 
# Rename ordinal outcome to distinguish as 3-category version
Example2$apply3=Example2$apply

# Create new binary outcome JUST to demonstrate logistic regression
Example2$apply2 = NA  # New empty variable
Example2$apply2[which(Example2$apply==0)]=0 
Example2$apply2[which(Example2$apply>0)]=1

# Rename and center predictors
Example2$parD=Example2$pared
Example2$gpa3=Example2$gpa-3

# Recode ref for public to create positive slope
Example2$priv = NA  # New empty variable
Example2$priv[which(Example2$public==1)]=0 
Example2$priv[which(Example2$public==0)]=1

# Label variables as comments only (not actually added to data)
#apply3= "apply3: 0=Not, 1=Eh, 2=Very"
#apply2= "apply2: 0=No, 1=Pry"
#parD=   "parD: Parent Has Graduate Degree (0=N,1=Y)"
#priv=   "priv: Student Attends Private University (0=N,1=Y)"
#gpa3=   "gpa3: Student GPA (0=3)" 

# Filter to only cases complete on all variables to be used below
Example2 = Example2[complete.cases(Example2[ c("apply3","parD","priv","gpa3")]),]


######################################################################################
#####                  BEGIN EXAMPLE 2a BINARY OUTCOME MODELS                    #####
######################################################################################

# Open external file to save results to for Example 2a
txtStart(file=paste0(filesave,"PSQF6270_Example2a_R_Output.txt"))

# Subset data to just needed columns for Example 2a for correlations
Example2a = Example2[c("apply2","parD","priv","gpa3")]

print("R Descriptive Statistics and Pearson Correlations for Original Variables")
describe(x=Example2a)
cor(x=Example2a) # Pearson correlations

# Recognize categorical variables as factor variables
Example2a$apply2 = as.factor(Example2a$apply2)
Example2a$parD   = as.factor(Example2a$parD)
Example2a$priv   = as.factor(Example2a$priv)
print("hetcor determines correlation type based on variable type")
hetcor(data=Example2a, ML=TRUE, std.err=TRUE, use="pairwise.complete.obs")

txtComment(" ") # insert blank space in output  

######################################################################################
print("R Empty Model Predicting Binary Apply2")
Model2Empty = glm(data=Example2, family=binomial(link="logit"), formula=apply2~1)
summary(Model2Empty); BIC(Model2Empty) # Get BIC too
print("Convert logits to probability to check interpretation")
Model2EmptyProb=1/(1+exp(-1*coefficients(Model2Empty))); Model2EmptyProb 

txtComment(" ") # insert blank space in output 

######################################################################################
print("R Main-Effects Model Predicting Binary Apply2")
Model2Main = glm(data=Example2, family=binomial(link="logit"), 
                 formula=apply2~1+gpa3+parD+priv)
summary(Model2Main); BIC(Model2Main) # Get BIC too
confint.default(Model2Main) # Get 95% CIs for logit parameters

print("Likelihood Ratio Test of Predictors")
print("Analogous to F-test for model R2 in general LM")
anova(Model2Empty, Model2Main, test="LRT") # Nested "fewer" model goes first

print("Pseudo-R2 values"); PseudoR2(x=Model2Main, which="all")

print("Odds ratios with 95% CIs using standard errors")
OddsRatio(x=Model2Main, conf.level=.95, digits=5, use.profile=FALSE)

# If the OddsRatio function doesn't work, use this instead
exp(cbind(OR=coef(Model2Main), confint(Model2Main)))

print("Example yhat in logits for specific values of predictors") 
Main2Logits = prediction(model=Model2Main, type="link",
                         at=list(gpa3=-1:1, parD=0, priv=0))
summary(Main2Logits)
print("Example yhat in probability for specific values of predictors") 
Main2Probs = prediction(model=Model2Main, type="response", 
                        at=list(gpa3=-1:1, parD=0, priv=0))
summary(Main2Probs)

txtComment(" ") # insert blank space in output 

######################################################################################
print("R Interaction Model Predicting Binary Apply2")
Model2Int = glm(data=Example2, family=binomial(link="logit"), 
                formula=apply2~1+gpa3+parD+priv+gpa3:parD+gpa3:priv)
summary(Model2Int); BIC(Model2Int) # Get BIC too
confint.default(Model2Int) # Get 95% CIs for logit parameters

print("Simple slopes for GPA by moderators")
Int2Slopes = (summary(glht(model=Model2Int, linfct=rbind(
  "GPA Slope: Ndeg Pub = B1      " = c(0,1,0,0,0,0),  # in order of fixed effects
  "GPA Slope: Ndeg Pri = B1+B5   " = c(0,1,0,0,0,1),
  "GPA Slope: Ydeg Pub = B1+B4   " = c(0,1,0,0,1,0),
  "GPA Slope: Ydeg Pri = B1+B4+B5" = c(0,1,0,0,1,1))),test=adjusted("none")))
Int2Slopes

print("Likelihood Ratio Test of 2 New Interactions")
print("Analogous to F-test for R2 change in general LM")
anova(Model2Main, Model2Int, test="LRT") # Nested "fewer" model goes first

print("Odds ratios with 95% CIs using standard errors")
OddsRatio(x=Model2Int, conf.level=.95, digits=5, use.profile=FALSE)

# If the OddsRatio function doesn't work, use this instead
exp(cbind(OR=coef(Model2Int), confint(Model2Int)))

print("Odds ratios for simple slopes using GLHT output")
data.frame(OR=exp(Int2Slopes$test$coefficients))

print("Yhat in logits for specific values of predictors") 
Int2Logits = prediction(model=Model2Int, type="link",
                        at=list(gpa3=-1:1,parD=0:1,priv=0:1))
summary(Int2Logits)
print("Yhat in probability for specific values of predictors") 
Int2Probs = prediction(model=Model2Int, type="response", 
                       at=list(gpa3=-1:1,parD=0:1,priv=0:1))
summary(Int2Probs)

txtComment(" ") # insert blank space in output 

######################################################################################
print("R Interaction Model Predicting Binary Apply2")
print("Using Probit Instead of Logit")
Model2IntProbit =glm(data=Example2, family=binomial(link="probit"), 
                     formula=apply2~1+gpa3+parD+priv+gpa3:parD+gpa3:priv)
summary(Model2IntProbit); BIC(Model2IntProbit) # Get BIC too

print("Yhat in probits for specific values of predictors") 
Int2Probits = prediction(model=Model2IntProbit, type="link",
                        at=list(gpa3=-1:1,parD=0:1,priv=0:1))
summary(Int2Probits)
print("Yhat in probability for specific values of predictors") 
Int2ProbitProbs = prediction(model=Model2IntProbit, type="response", 
                             at=list(gpa3=-1:1,parD=0:1,priv=0:1))
summary(Int2ProbitProbs)

#####################################################################################
# Close external file of saved results for Example 2a
txtStop()


######################################################################################
#####                BEGIN EXAMPLE 2b CATEGORICAL OUTCOME MODELS                 #####
######################################################################################

# Open external file to save results to for Example 2b
txtStart(file=paste0(filesave,"PSQF6270_Example2b_R_Output.txt"))

# Subset data to just needed columns for Example 2a for correlations
Example2b = Example2[c("apply3","parD","priv","gpa3")]

print("R Pearson Correlations for Original Variables")
cor(x=Example2b) # Pearson correlations

# Recognize categorical variables as factor variables
Example2b$apply3 = as.factor(Example2b$apply3)
Example2b$parD   = as.factor(Example2b$parD)
Example2b$priv   = as.factor(Example2b$priv)
print("hetcor determines correlation type based on variable type")
hetcor(data=Example2b, ML=TRUE, std.err=TRUE, use="pairwise.complete.obs")

txtComment(" ") # insert blank space in output  

print("R Proportions for Ordinal Apply")
prop.table(table(x=Example2$apply3))

txtComment(" ") # insert blank space in output  


############################ BEGIN EMPTY ORDINAL MODELS ##############################

print("R Empty Model Predicting Ordinal Apply3")
Model3Empty = vglm(data=Example2, family=cumulative(link="logitlink",reverse=TRUE,parallel=TRUE),
                   formula=apply3~1)
summary(Model3Empty); AIC(Model3Empty); BIC(Model3Empty) # Get AIC and BIC too
print("Convert logits to probability to check interpretation")
Model3EmptyProb=1/(1+exp(-1*coefficients(Model3Empty))); Model3EmptyProb 

txtComment(" ") # insert blank space in output 


##################### BEGIN PROPORTIONAL ODDS ORDINAL MODELS #########################

print("R Proportional Odds Model Predicting Ordinal Apply3")
Model3PO = vglm(data=Example2, family=cumulative(link="logitlink",reverse=TRUE,parallel=TRUE), 
                formula=apply3~1+gpa3+parD+priv)
summary(Model3PO); AIC(Model3PO); BIC(Model3PO) # Get AIC and BIC too

print("Likelihood Ratio Test of Predictors")
print("Analogous to F-test for model R2 in general LM")
anova(Model3Empty, Model3PO, type=1) # Nested "fewer" model goes first

print("Odds ratios with 95% CIs")
exp(cbind(OR = coefficients(Model3PO), confint.default(Model3PO)))

txtComment(" ") # insert blank space in output 


################# BEGIN NON/PARTIAL PROPORTIONAL ODDS ORDINAL MODELS ####################

print("R Non-Proportional Odds Model Predicting Ordinal Apply3")
Model3NPO = vglm(data=Example2, family=cumulative(link="logitlink",reverse=TRUE,parallel=FALSE), 
                 formula=apply3~1+gpa3+parD+priv)
summary(Model3NPO); AIC(Model3NPO); BIC(Model3NPO) # Get AIC and BIC too

print("Likelihood Ratio Test for Overall Proportional Odds")
anova(Model3PO, Model3NPO, type=1) # Nested "fewer" model goes first

print("Univ Wald tests of submodel slope differences")
NPOuniv = (summary(glht(model=Model3NPO, linfct=rbind(
  "gpa3 slope diff" = c(0,0,-1,1, 0,0, 0,0),  # in order of fixed effects
  "parD slope diff" = c(0,0, 0,0,-1,1, 0,0),
  "priv slope diff" = c(0,0, 0,0, 0,0,-1,1))),test=adjusted("none"))); NPOuniv

txtComment(" ") # insert blank space in output 


print("R Partial Proportional Odds Model Predicting Ordinal Apply3")
Model3CPO = vglm(data=Example2, family=cumulative(link="logitlink",reverse=TRUE,parallel=FALSE~priv), 
                 formula=apply3~1+gpa3+parD+priv)
summary(Model3CPO); AIC(Model3CPO); BIC(Model3CPO) # Get AIC and BIC too

print("Univ Wald test of submodel slope difference")
CPOuniv = (summary(glht(model=Model3CPO, linfct=rbind(
  "priv Slope PO" = c(0,0,0,0,-1,1))),test=adjusted("none"))); CPOuniv

print("Odds ratios with 95% CIs")
exp(cbind(OR = coefficients(Model3CPO), confint.default(Model3CPO)))

# Create fake people for use in generating predicted outcomes
FakeGpa3 = c(-1,0,1,-1,0,1,-1,0,1,-1,0,1)
FakeParD = c( 0,0,0, 0,0,0, 1,1,1, 1,1,1)
FakePriv = c( 0,0,0, 1,1,1, 0,0,0, 1,1,1)
# Create dataset using just-created columns and constants for other model variables
FP = data.frame(gpa3=FakeGpa3, parD=FakeParD, priv=FakePriv)

print("Get Yhat for specific values of predictors in fake people") 
print("Y column = predicted yhat, Yprob = predicted probability")
PredCPO = data.frame(FP, Y=predict(object=Model3CPO, newdata=FP, type="link"),
                     Yprob=predict(object=Model3CPO, newdata=FP, type="response"))
print("Rename columns into something meaningful")
names(PredCPO)[names(PredCPO)=='Y.logitlink.P.Y..2..']='YlogitGT0'
names(PredCPO)[names(PredCPO)=='Y.logitlink.P.Y..3..']='YlogitGT1'; PredCPO

txtComment(" ") # insert blank space in output 


########################### BEGIN NOMINAL MODELS ####################################

print("R Empty Model Predicting Nominal Apply3 -- ref is SECOND category of y=1")
Model3NomEmpty = vglm(data=Example2, family=multinomial(refLevel=2),reverse=TRUE, 
                      formula=apply3~1)
summary(Model3NomEmpty); AIC(Model3NomEmpty); BIC(Model3NomEmpty) # Get AIC and BIC too
print("Convert logits to probability to check interpretation")
Model3NomEmptyProb=1/(1+exp(-1*coefficients(Model3NomEmpty))); Model3NomEmptyProb 

txtComment(" ") # insert blank space in output 


print("R 3-Predictor Model Predicting Nominal Apply3 -- ref is SECOND category of y=1")
Model3NomMain = vglm(data=Example2, family=multinomial(refLevel=2),reverse=TRUE, 
                     formula=apply3~1+gpa3+parD+priv)
summary(Model3NomEmpty); AIC(Model3NomMain); BIC(Model3NomMain) # Get AIC and BIC too

print("Univ Wald tests of submodel slope differences after reversing sign of 0-model slopes")
NomUniv = (summary(glht(model= Model3NomMain, linfct=rbind(
  "gpa3 slope diff" = c(0,0, 1,1, 0,0, 0,0),  # in order of fixed effects
  "parD slope diff" = c(0,0, 0,0, 1,1, 0,0),
  "priv slope diff" = c(0,0, 0,0, 0,0, 1,1))),test=adjusted("none"))); NomUniv

print("Likelihood Ratio Test of Predictors")
print("Analogous to F-test for model R2 in general LM")
anova(Model3Empty, Model3NomMain, type=1) # Nested "fewer" model goes first

print("Odds ratios with 95% CIs")
exp(cbind(OR = coefficients(Model3NomMain), confint.default(Model3NomMain)))

# Create fake people for use in generating predicted outcomes
FakeGpa3 = c(-1,0,1,-1,0,1,-1,0,1,-1,0,1)
FakeParD = c( 0,0,0, 0,0,0, 1,1,1, 1,1,1)
FakePriv = c( 0,0,0, 1,1,1, 0,0,0, 1,1,1)
# Create dataset using just-created columns and constants for other model variables
FP = data.frame(gpa3=FakeGpa3, parD=FakeParD, priv=FakePriv)

print("Get Yhat for specific values of predictors in fake people") 
print("Y column = predicted yhat, Yprob = predicted probability")
PredNom = data.frame(FP, Y=predict(object=Model3NomMain, newdata=FP, type="link"),
                     Yprob=predict(object=Model3NomMain, newdata=FP, type="response"))
print("Rename columns into something meaningful")
names(PredNom)[names(PredNom)=='Y.log.mu..1..mu..2..']='Ylogit1vs0'
names(PredNom)[names(PredNom)=='Y.log.mu..3..mu..2..']='Ylogit1vs2'
PredNom

txtComment(" ") # insert blank space in output 

################################################################################################
# Close external file of saved results for Example 2b
txtStop()

